<?php

namespace EventPortal\Tests\Unit\EventDelegate\Handlers;

use EventPortal\Domain\Delegate\Models\DelegateFormStep;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\EventDelegate\Handlers\UpdateProgress;
use EventPortal\Domain\Form\Payload;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Pipeline\Pipeline;

class UpdateProgressTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_create_delegate_progress(): void
    {
        $event = factory(Event::class)->create();
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $step = factory(FormStep::class)->create(['form_id' => $form->id]);
        $eventDelegate = factory(EventDelegate::class)->create(['event_id' => $event->id]);

        $payload = Payload::make()
            ->withContext(['eventDelegate' => $eventDelegate, 'formStep' => $step]);

        app(Pipeline::class)
            ->send($payload)
            ->through([new UpdateProgress])
            ->thenReturn();

        $this->assertDatabaseHas('delegate_form_step', [
            'delegate_id' => $eventDelegate->delegate->id,
            'form_step_id' => $step->id,
        ]);
        $delegateFormStep = DelegateFormStep::first();
        $this->assertSameModel($eventDelegate->delegate, $delegateFormStep->delegate);
        $this->assertSameModel($step, $delegateFormStep->step);
    }

    /** @test **/
    public function can_update_delegate_progress(): void
    {
        $event = factory(Event::class)->create();
        $form = factory(Form::class)->create(['event_id' => $event->id]);
        $step = factory(FormStep::class)->create(['form_id' => $form->id]);
        $eventDelegate = factory(EventDelegate::class)->create(['event_id' => $event->id]);
        factory(DelegateFormStep::class)
            ->state('partial')
            ->create(['delegate_id' => $eventDelegate->delegate->id, 'form_step_id' => $step->id]);

        $payload = Payload::make()
            ->withContext(['eventDelegate' => $eventDelegate, 'formStep' => $step]);

        app(Pipeline::class)
            ->send($payload)
            ->through([new UpdateProgress])
            ->thenReturn();

        $this->assertCount(1, DelegateFormStep::all());
    }
}
