<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\EventSpeaker;
use EventPortal\Domain\Util\RelationshipType;
use EventPortal\Tests\TestCase;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Foundation\Testing\RefreshDatabase;

class RelationshipTypeTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_get_type()
    {
        $delegate = factory(Delegate::class)->make();

        $type = RelationshipType::make($delegate, 'events');

        $this->assertEquals(BelongsToMany::class, $type->getType());
    }

    /** @test **/
    public function can_get_type_when_nested()
    {
        $eventSpeaker = factory(EventSpeaker::class)->make();

        $type = RelationshipType::make($eventSpeaker, 'speaker.user');

        $this->assertEquals(BelongsTo::class, $type->getType());
    }

    /** @test **/
    public function can_assert_it_is_belongs_to_many()
    {
        $delegate = factory(Delegate::class)->make();

        $type = RelationshipType::make($delegate, 'events');

        $this->assertTrue($type->isBelongsToMany());
    }
}
